@@ -1,4 +1,4 @@
-from bgameb.items import Dice, Card, Step
+from bgameb.items import Dice, Card, Step_
 from bgameb.tools import Shaker, Deck, Steps, Bag
 from bgameb.players import Player
 from bgameb.game import Game
