@@ -1,46 +1,31 @@
 """Game tools classes
 """
 import random
+from pydantic import Field, PositiveInt
 from collections import deque
+from collections.abc import KeysView
 from heapq import heappop, heappush
-from typing import Optional, Iterable
-from dataclasses import dataclass, field, replace
-from dataclasses_json import (
-    config, DataClassJsonMixin, dataclass_json, Undefined
-        )
+from typing import Optional, Iterable, Union, Any
 from bgameb.base import Base, Component
-from bgameb.items import Card, Dice, Step, BaseItem
+from bgameb.items import Card, Dice, Step_, BaseItem
 from bgameb.errors import ArrangeIndexError, ComponentClassError
 
 
-@dataclass_json(undefined=Undefined.INCLUDE)
-@dataclass(repr=False)
-class BaseTool(Base, DataClassJsonMixin):
+class BaseTool(Base):
     """Base class for game tools (like decks or shakers)
     """
-    current: list[BaseItem] = field(
-        default_factory=list,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
+    c: Component[str, BaseItem] = Field(
+        default_factory=Component, exclude=True, repr=False
             )
+    current: list[BaseItem] = []
+    last: Optional[BaseItem] = None
 
-    def __post_init__(self) -> None:
-        super().__post_init__()
-
-    def _item_replace(self, item: BaseItem) -> BaseItem:
-        """Replace item in a current
-
-        Returns:
-            Item (BaseItem): an item object
-        """
-        return replace(item)
-
-    def clear(self) -> None:
-        """Clear the current bag
-        """
-        self.current.clear()
-        self._logger.debug('Current clear!')
+    class Config(Base.Config):
+        json_encoders = {
+            Component: lambda c: c.to_json()
+                }
 
+    @property
     def current_ids(self) -> list[str]:
         """Get ids of current objects
 
@@ -49,6 +34,51 @@ class BaseTool(Base, DataClassJsonMixin):
         """
         return [item.id for item in self.current]
 
+    @property
+    def last_id(self) -> Optional[str]:
+        """Get id of last
+
+        Returns:
+            Optional[str]: id
+        """
+        if self.last is not None:
+            return self.last.id
+        return None
+
+    def get_items(self) -> dict[str, BaseItem]:
+        """Get items from Component
+
+        Returns:
+            dict[str, BaseItem]: items mapping
+        """
+        return {item.id: item for item in self.c.values()}
+
+    def _item_replace(self, item: BaseItem) -> BaseItem:
+        """Get item replaced copy
+
+        Returns:
+            Item (BaseItem): an item object
+        """
+        return item.__class__(**item.dict())
+
+    def by_id(self, id: str) -> list[BaseItem]:
+        """Get item from current by its id
+
+        Args:
+            id (str): item id
+
+        Returns:
+            list[BaseItem]: items
+        """
+        return [item for item in self.current if item.id == id]
+
+    def clear(self) -> None:
+        """Clear the current and last
+        """
+        self.current.clear()
+        self.last = None
+        self._logger.debug('Current and last clear!')
+
     def append(self, item: BaseItem) -> None:
         """Append item to current
 
@@ -60,7 +90,7 @@ class BaseTool(Base, DataClassJsonMixin):
         self._logger.debug(f'To current is appended item: {item.id}')
 
     def count(self, item_id: str) -> int:
-        """Count the number of current items with by id.
+        """Count the number of current items with given id.
 
         Args:
             item_id (str: an item id
@@ -68,7 +98,7 @@ class BaseTool(Base, DataClassJsonMixin):
         Returns:
             int: count of items
         """
-        count = self.current_ids().count(item_id)
+        count = self.current_ids.count(item_id)
         self._logger.debug(f'Count of {item_id} in current is {count}')
         return count
 
@@ -91,8 +121,8 @@ class BaseTool(Base, DataClassJsonMixin):
         start: int = 0,
         end: Optional[int] = None
             ) -> int:
-        """Return the position of item_id in the current
-        (at or after index start and before index stop).
+        """Return the position of item in the current
+        (after index start and before index stop).
         Returns the first match or raises ValueError if not found.
 
         Args:
@@ -103,14 +133,14 @@ class BaseTool(Base, DataClassJsonMixin):
         Returns:
             int: index of the the first match
         """
-        names = self.current_ids()
+        names = self.current_ids
         ind = names.index(item_id, start) if end is None \
             else names.index(item_id, start, end)
         self._logger.debug(f'Index of {item_id} in current is {ind}')
         return ind
 
     def insert(self, item: BaseItem, pos: int) -> None:
-        """Insert item into the current at position pos.
+        """Insert item into the current at given position.
 
         Args:
             item (Item): an item object
@@ -122,14 +152,14 @@ class BaseTool(Base, DataClassJsonMixin):
 
     def pop(self) -> BaseItem:
         """Remove and return an item from the current.
-        If no cards are present, raises an IndexError.
+        If no items are present, raises an IndexError.
 
         Returns:
             Item: an item object
         """
-        item = self.current.pop()
-        self._logger.debug(f'{item.id} is poped from current')
-        return item
+        self.last = self.current.pop()
+        self._logger.debug(f'{self.last.id} is poped from current')
+        return self.last
 
     def remove(self, item_id: str) -> None:
         """Remove the first occurrence of item from current.
@@ -143,58 +173,29 @@ class BaseTool(Base, DataClassJsonMixin):
         self._logger.debug(f'Is removed from current {item_id}')
 
     def reverse(self) -> None:
-        """Reverse the items of the current.
+        """Reverse the items in the current.
         """
         self.current.reverse()
         self._logger.debug('Current is reversed')
 
 
-@dataclass_json(undefined=Undefined.INCLUDE)
-@dataclass(repr=False)
-class Bag(BaseTool, DataClassJsonMixin):
+class Bag(Base):
     """Bag object
+
+    Attr:
+        - c (Component[BaseItem]) - items mapping
     """
-    c: Component[BaseItem] = field(
-        default_factory=Component,
-        metadata=config(exclude=lambda x: True),  # type: ignore
+    c: Component[str, BaseItem] = Field(
+        default_factory=Component, exclude=True, repr=False
             )
 
-    def __post_init__(self) -> None:
-        super().__post_init__()
-        self.c = Component()
-
-    def get_items(self) -> dict[str, BaseItem]:
-        return {
-            key: val for key, val
-            in self.c.items()
-            if issubclass(val.__class__, BaseItem)
+    class Config(Base.Config):
+        json_encoders = {
+            Component: lambda c: c.to_json()
                 }
 
-    def deal(self, items: Optional[list[str]] = None) -> 'Bag':
-        """Deal new bag current
-
-        Args:
-            items (Optional[List[str]]): list of items ids
-
-        Returns:
-            Bag
-        """
-        self.clear()
-
-        if not items:
-            for stuff in self.c.values():
-                self.append(stuff)
-        else:
-            for id in items:
-                comp = self.c.by_id(id)
-                if comp:
-                    self.append(comp)
-
-        self._logger.debug(f'Is deal current: {self.current_ids()}')
-        return self
-
     def add(self, stuff: BaseItem) -> None:
-        """Add stuff to component
+        """Add stuff to Bag component
 
         Args:
             stuff (BaseItem): game stuff
@@ -208,27 +209,43 @@ class Bag(BaseTool, DataClassJsonMixin):
             raise ComponentClassError(stuff, self._logger)
 
 
-@dataclass_json(undefined=Undefined.INCLUDE)
-@dataclass(repr=False)
-class Shaker(BaseTool, DataClassJsonMixin):
+class Shaker(BaseTool):
     """Create shaker for roll dices or flip coins
-    """
-    current: list[Dice] = field(
-        default_factory=list,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
-        )
-    c: Component[Dice] = field(
-        default_factory=Component,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-            )
 
-    def __post_init__(self) -> None:
-        super().__post_init__()
-        self.c = Component()
+    Attr:
+        - c (Component[Dice]): dices mapping
+        - current (Deque[Dice]): current dice deque.
+        - last (Optional[Dice]): last poped from current.
+        - last_roll (Optional[dict[str, list[PositiveInt]]]): last roll result.
+        - last_roll_mapped (Optional[dict[str, list[Any]]]): last mapped
+                                                             roll result.
+    """
+    c: Component[str, Dice] = Field(  # type: ignore
+        default_factory=Component, exclude=True, repr=False
+            )
+    current: list[Dice] = []  # type: ignore
+    last: Optional[Dice] = None
+    last_roll: dict[str, list[PositiveInt]] = {}
+    last_roll_mapped: dict[str, list[Any]] = {}
+
+    def add(self, stuff: Dice) -> None:
+        """Add dice to component
+
+        Args:
+            stuff (Dice): dice object
+        """
+        if isinstance(stuff.__class__, Dice) \
+                or issubclass(stuff.__class__, Dice):
+            self.c.update(stuff)
+            self._logger.info(
+                f'Component updated by stuff with id="{stuff.id}".'
+                    )
+        else:
+            raise ComponentClassError(stuff, self._logger)
 
     def deal(self, items: Optional[list[str]] = None) -> 'Shaker':
-        """Deal new shaker current
+        """Deal new shaker current. The current is cleared
+        before deal.
 
         Args:
             items (Optional[List[str]]): items ids
@@ -243,58 +260,56 @@ class Shaker(BaseTool, DataClassJsonMixin):
                 self.append(stuff)
         else:
             for id in items:
-                if id in self.c.ids():
+                if id in self.c.ids:
                     self.append(self.c[id])
 
-        self._logger.debug(f'Is deal current: {self.current_ids()}')
+        self._logger.debug(f'Is deal current: {self.current_ids}')
         return self
 
     def roll(self) -> dict[str, list[int]]:
         """Roll all stuff in shaker and return results
 
         Return:
-            Dict[str, Tuple[int]]: result of roll
+            Dict[str, list[int]]: result of roll
 
         .. code-block::
             :caption: Example:
 
                 {
-                    "six_dice": (5, 3, 2, 5),
-                    "twenty_dice": {2, 12, 4},
+                    "six_dice": [5, 3, 2, 5],
+                    "twenty_dice": [2, 12, 4],
                 }
         """
-        roll = {}
+        self.last_roll = {}
 
         for item in self.current:
-            roll[item.id] = item.roll()
+            self.last_roll[item.id] = item.roll()
 
-        self._logger.debug(f'Result of roll: {roll}')
+        self._logger.debug(f'Result of roll: {self.last_roll}')
 
-        return roll
+        return self.last_roll
 
-    def add(self, stuff: Dice) -> None:
-        """Add stuff to component
+    def roll_mapped(self) -> dict[str, list[Any]]:
+        """Roll all stuff in shaker and return mapped results.
+        If any stuff unmaped - empty list returned for this item.
 
-        Args:
-            stuff (Dice): game stuff
+        Returns:
+            dict[str, list[Any]]: result of roll
         """
-        if isinstance(stuff.__class__, Dice) \
-                or issubclass(stuff.__class__, Dice):
-            self.c.update(stuff)
-            self._logger.info(
-                f'Component updated by stuff with id="{stuff.id}".'
-                    )
-        else:
-            raise ComponentClassError(stuff, self._logger)
+        self.last_roll_mapped = {}
+
+        for item in self.current:
+            self.last_roll_mapped[item.id] = item.roll_mapped()
+
+        self._logger.debug(f'Result of roll: {self.last_roll_mapped}')
+
+        return self.last_roll_mapped
 
 
-@dataclass_json(undefined=Undefined.INCLUDE)
-@dataclass(repr=False)
-class Deck(BaseTool, DataClassJsonMixin):
+class Deck(BaseTool):
     """Deck object
 
-    Deck ia a Bag subclass that contains Cards for
-    define curent game deque.
+    Deck contains Cards for define curent game deque.
 
     You can add cards, define it counts and deal a deck.
     Result is saved in current deck attr as deque object. This object
@@ -303,34 +318,38 @@ class Deck(BaseTool, DataClassJsonMixin):
     <https://docs.python.org/3/library/collections.html#deque-objects>`_
 
     Attr:
+        - c (Component[Card]): components of Deck
         - current (Deque[Card]): current cards deque.
+        - last (Optional[Card]): last poped from current.
 
     .. code-block::
         :caption: Example:
 
             deque(Card1, Card3, Card2, Card4)
     """
-    current: deque[Card] = field(
-        default_factory=deque,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
-        )
-    c: Component[Card] = field(
-        default_factory=Component,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-            )
-    last: Optional[Card] = field(
-        default=None,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
+    c: Component[str, Card] = Field(  # type: ignore
+        default_factory=Component, exclude=True, repr=False
             )
+    current: deque[Card] = Field(default_factory=deque)  # type: ignore
+    last: Optional[Card] = None
 
-    def __post_init__(self) -> None:
-        super().__post_init__()
-        self.c = Component()
+    def add(self, stuff: Card) -> None:
+        """Add card to component
+
+        Args:
+            stuff (Card): Card object
+        """
+        if isinstance(stuff.__class__, Card) \
+                or issubclass(stuff.__class__, Card):
+            self.c.update(stuff)
+            self._logger.info(
+                f'Component updated by stuff with id="{stuff.id}".'
+                    )
+        else:
+            raise ComponentClassError(stuff, self._logger)
 
     def _item_replace(self, item: Card) -> Card:  # type: ignore[override]
-        """Replace item in a current
+        """Get replaced copy of card
 
         Args:
             item (Card): a card object
@@ -338,16 +357,44 @@ class Deck(BaseTool, DataClassJsonMixin):
         Returns:
             Card: a card object
         """
-        item = replace(item)
-        del item.count
+        item = item.__class__(**item.dict())
+        item.count = 1
         return item
 
-    def clear(self) -> None:
-        """Clear the current and last of Deck
+    def deal(self, items: Optional[list[str]] = None) -> 'Deck':
+        """Deal new deck current. Cured is cleared
+        before deal.
+
+        Args:
+            items (Optional[List[str]]): list of cards ids
+
+        Returns:
+            Deck
         """
-        self.current.clear()
-        self.last = None
-        self._logger.debug('Current and last are clear!')
+        self.clear()
+
+        if not items:
+            for stuff in self.c.values():
+                for _ in range(stuff.count):
+                    self.append(stuff)
+        else:
+            for id in items:
+                comp = self.c.by_id(id)
+                if comp:
+                    self.append(comp)
+
+        self._logger.debug(f'Is deal current: {self.current_ids}')
+        return self
+
+    def shuffle(self) -> 'Deck':
+        """Random shuffle current deck.
+
+        Returns:
+            Deck
+        """
+        random.shuffle(self.current)
+        self._logger.debug(f'Is shuffled: {self.current_ids}')
+        return self
 
     def appendleft(self, item: Card) -> None:
         """Add card to the left side of the current deck.
@@ -374,17 +421,6 @@ class Deck(BaseTool, DataClassJsonMixin):
             f'Current are extended by {[item.id for item in items]} from left'
                 )
 
-    def pop(self) -> Card:
-        """Remove and return an item from the current.
-        If no cards are present, raises an IndexError.
-
-        Returns:
-            Item: an item object
-        """
-        self.last = self.current.pop()
-        self._logger.debug(f'{self.last.id} is poped from current')
-        return self.last
-
     def popleft(self) -> Card:
         """Remove and return a card from the left side of the current deck.
         If no cards are present, raises an IndexError.
@@ -406,102 +442,147 @@ class Deck(BaseTool, DataClassJsonMixin):
         self.current.rotate(n)
         self._logger.debug(f'Current is rotate by {n}')
 
-    def deal(self, items: Optional[list[str]] = None) -> 'Deck':
-        """Deal new deck current
+    def _check_order_len(self, len_: int) -> None:
+        """Check is order len valid
 
         Args:
-            items (Optional[List[str]]): list of cards ids
+            l (int): len of order of cards
 
-        Returns:
-            Deck
+        Raises:
+            ArrangeIndexError: Is given empty order
+            ArrangeIndexError: The len of current deque not match order len
         """
-        self.clear()
-
-        if not items:
-            for stuff in self.c.values():
-                for _ in range(stuff.count):
-                    self.append(stuff)
-        else:
-            for id in items:
-                comp = self.c.by_id(id)
-                if comp:
-                    self.append(comp)
-
-        self._logger.debug(f'Is deal current: {self.current_ids()}')
-        return self
-
-    def shuffle(self) -> 'Deck':
-        """Random shuffle current deck
-
-        Returns:
-            Deck
-        """
-        random.shuffle(self.current)
-        self._logger.debug(f'Is shuffled: {self.current}')
-        return self
-
-    def to_arrange(
-        self,
-        start: int,
-        end: int
-            ) -> tuple[
-                list[Card],
-                tuple[list[Card], list[Card]]
-                    ]:
-        """Prepare current deck to arrange
-
-        Args:
-            start (int): start of slice
-            end (int): end of slice
-
-        Start and end cant be less than 0 and end must be greater than start.
-        Arranged deck are splited to two part - center (used for arrange part
-        of deck) and tupple with last part of deck. You can rearrange center
-        part and then concatenate it with last part in new deque by arrange()
-        method.
-
-        Return:
-            Tuple[List[Card], Tuple[List[Card]]: parts to arrange
-        """
-        if start < 0 or end < 0 or end < start:
+        if not len_:
             raise ArrangeIndexError(
-                message=f'Nonpositive or broken {start=} or {end=}',
+                'Given empty order',
                 logger=self._logger
                     )
-        to_split = list(self.current)
-        splited = (to_split[start:end], (to_split[0:start], to_split[end:]))
-        self._logger.debug(f'To arrange result: {splited}')
 
-        return splited
+        if len_ > len(self.current):
+            raise ArrangeIndexError(
+                f'The len of current deque is {len(self.current)} '
+                f'but given order has len {len_}.',
+                logger=self._logger
+                    )
 
-    def arrange(
+    def _check_is_to_arrange_valid(
         self,
-        arranged: list[Card],
-        last: tuple[list[Card], list[Card]]
+        order: list[str],
+        to_arrange: Union[list[str], KeysView[str]]
+            ) -> None:
+        """Chek is order and deque contains same elements
+
+        Args:
+            order (List[str]): ordered list of cards ids
+            to_arrange (list[str]): list of deque ids
+
+        Raises:
+            ArrangeIndexError: Given card ids and deque ids not match
+        """
+        if set(to_arrange) ^ set(order):
+            raise ArrangeIndexError(
+                'Given card ids and deque ids not match.',
+                logger=self._logger
+                    )
+
+    def reorder(
+        self,
+        order: list[str],
             ) -> 'Deck':
-        """Concatenate new current deck from given arranged list and last of
-        deck. Use to_arrange() method to get list to arrange and last.
+        """Reorder current deque from right side.
 
         Args:
-            arranged (List[Card]): arranged list of cards
-            last: (Tuple[List[Card], List[Card]]): last of deck
+            order (List[str]): ordered list of cards ids
+            ordered from left side to right
 
         Returns:
             Deck
         """
-        reorranged: deque = deque()
-        reorranged.extend(last[0])
-        reorranged.extend(arranged)
-        reorranged.extend(last[1])
+        len_ = len(order)
+        self._check_order_len(len_)
 
-        if len(reorranged) == len(self.current):
-            self.current = reorranged
-            self._logger.debug(f'Arrange result: {reorranged}')
-        else:
+        to_arrange = {
+            self.current[len_-ind-1].id: self.current[len_-ind-1]
+            for ind in range(len_)
+                }
+        self._check_is_to_arrange_valid(order, to_arrange.keys())
+
+        for _ in range(len_):
+            self.pop()
+
+        for card in order:
+            self.append(to_arrange[card])
+
+        self._logger.debug(f'Is reordered right side of deque: {order}')
+
+        return self
+
+    def reorderleft(
+        self,
+        order: list[str],
+            ) -> 'Deck':
+        """Reorder current deque from left side.
+
+        Args:
+            order (List[str]): ordered list of cards ids
+            ordered from left side to right
+
+        Returns:
+            Deck
+        """
+        len_ = len(order)
+        self._check_order_len(len_)
+
+        to_arrange = {
+            self.current[ind].id: self.current[ind]
+            for ind in range(len_)
+                }
+        self._check_is_to_arrange_valid(order, to_arrange.keys())
+
+        for _ in range(len_):
+            self.popleft()
+
+        for card in reversed(order):
+            self.appendleft(to_arrange[card])
+
+        self._logger.debug(f'Is reordered left side of deque: {order}')
+
+        return self
+
+    def reorderfrom(
+        self,
+        order: list[str],
+        start: int,
+            ) -> 'Deck':
+        """Reorder current deque from right side started
+        with given position.
+
+        Args:
+            start (int): start of reordering
+            order (List[str]): ordered list of cards ids
+            ordered from right side to left
+
+        Returns:
+            Deck
+        """
+        len_ = len(order)
+        self._check_order_len(len_)
+        if start <= 0 or start > len(self.current)-len_:
             raise ArrangeIndexError(
-                f'Wrong to_arranged parts: {arranged=}, {last=}',
+                'Given range is out of current index.',
                 logger=self._logger
                     )
+
+        old_deck = list(self.current)
+        to_arrange = {
+            card.id: card
+            for card in old_deck[start:start+len_]
+                }
+        self._check_is_to_arrange_valid(order, to_arrange.keys())
+
+        for ind1, ind2 in enumerate(range(start, start+len_)):
+            self.current[ind2] = to_arrange[order[ind1]]
+
         return self
 
     def search(
@@ -509,7 +590,7 @@ class Deck(BaseTool, DataClassJsonMixin):
         query: dict[str, int],
         remove: bool = True
             ) -> list[Card]:
-        """Search for cards in current deck
+        """Search for cards in current by its id.
 
         Args:
             query (Dict[str, int]): dict with id of searched
@@ -518,7 +599,7 @@ class Deck(BaseTool, DataClassJsonMixin):
                            current deck. Default to True.
 
         Return:
-            List[Card]: list of find cards
+            List[Card]: list of find cards, equal searching count
 
         .. code-block::
             :caption: Example:
@@ -589,14 +670,38 @@ class Deck(BaseTool, DataClassJsonMixin):
                     )
             return result
 
-    def add(self, stuff: Card) -> None:
-        """Add stuff to component
+
+class Steps(BaseTool):
+    """Game steps order object
+
+    Attr:
+        - c (Component[Step]): components of Steps.
+        - current (List[Tuple[int, Step]]): current order of steps.
+        - last (Step): last poped from current step.
+    """
+    c: Component[str, Step_] = Field(  # type: ignore
+        default_factory=Component, exclude=True, repr=False
+            )
+    current: list[tuple[int, Step_]] = []  # type: ignore
+    last: Optional[Step_] = None
+
+    @property
+    def current_ids(self) -> list[str]:
+        """Get ids of current objects
+
+        Returns:
+            List[str]: list ids of current
+        """
+        return [item[1].id for item in self.current]
+
+    def add(self, stuff: Step_) -> None:
+        """Add Step to component
 
         Args:
-            stuff (Card): game stuff
+            stuff (Step): Step object
         """
-        if isinstance(stuff.__class__, Card) \
-                or issubclass(stuff.__class__, Card):
+        if isinstance(stuff.__class__, Step_) \
+                or issubclass(stuff.__class__, Step_):
             self.c.update(stuff)
             self._logger.info(
                 f'Component updated by stuff with id="{stuff.id}".'
@@ -604,60 +709,6 @@ class Deck(BaseTool, DataClassJsonMixin):
         else:
             raise ComponentClassError(stuff, self._logger)
 
-
-@dataclass_json(undefined=Undefined.INCLUDE)
-@dataclass(repr=False)
-class Steps(BaseTool, DataClassJsonMixin):
-    """Game steps order object
-
-    Attr:
-        - current (List[Tuple[int, Step]]): current order of steps.
-        - last (Step): last pulled from current step
-    """
-    current: list[tuple[int, Step]] = field(
-        default_factory=list,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
-            )
-    last: Optional[Step] = field(
-        default=None,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-        repr=False,
-    )
-    c: Component[Step] = field(
-        default_factory=Component,
-        metadata=config(exclude=lambda x: True),  # type: ignore
-            )
-
-    def __post_init__(self) -> None:
-        super().__post_init__()
-        self.c = Component()
-
-    def clear(self) -> None:
-        """Clear the current and last of Steps
-        """
-        self.current.clear()
-        self.last = None
-        self._logger.debug('Current and last are clear!')
-
-    def push(self, item: Step) -> None:
-        """Push Step object to current
-
-        Args:
-            item (Step): Step class instance
-        """
-        replaced = replace(item)
-        heappush(self.current, (replaced.priority, replaced))
-
-    def pull(self) -> Step:
-        """Pull Step object from current with lowest priority
-
-        Returns:
-            Step: Step instance object
-        """
-        self.last = heappop(self.current)[1]
-        return self.last
-
     def deal(
         self,
         items: Optional[list[str]] = None
@@ -681,28 +732,58 @@ class Steps(BaseTool, DataClassJsonMixin):
                 if comp:
                     self.push(comp)
 
-        self._logger.debug(f'Is deal current: {self.current_ids()}')
+        self._logger.debug(f'Is deal current: {self.current_ids}')
         return self
 
-    def current_ids(self) -> list[str]:
-        """Get ids of current objects
-
-        Returns:
-            List[str]: list ids of current
-        """
-        return [item[1].id for item in self.current]
-
-    def add(self, stuff: Step) -> None:
-        """Add stuff to component
+    def by_id(self, id: str) -> list[BaseItem]:
+        """Get items from current by its id
 
         Args:
-            stuff (Step): game stuff
+            id (str): item id
+
+        Returns:
+            list[BaseItem]: items
         """
-        if isinstance(stuff.__class__, Step) \
-                or issubclass(stuff.__class__, Step):
-            self.c.update(stuff)
-            self._logger.info(
-                f'Component updated by stuff with id="{stuff.id}".'
-                    )
-        else:
-            raise ComponentClassError(stuff, self._logger)
+        return [item[1] for item in self.current if item[1].id == id]
+
+    def push(self, item: Step_) -> None:
+        """Push Step object to current
+
+        Args:
+            item (Step): Step class instance
+        """
+        replaced: Step_ = self._item_replace(item)  # type: ignore
+        heappush(self.current, (replaced.priority, replaced))
+
+    def pop(self) -> Step_:
+        """Pop Step object from current with lowest priority
+
+        Returns:
+            Step: Step instance object
+        """
+        self.last = heappop(self.current)[1]
+        self._logger.debug(f'{self.last.id} is poped from current')
+        return self.last
+
+    def append(self, item: BaseItem) -> None:
+        raise NotImplementedError
+
+    def extend(self, items: Iterable[BaseItem]) -> None:
+        raise NotImplementedError
+
+    def index(
+        self,
+        item_id: str,
+        start: int = 0,
+        end: Optional[int] = None
+            ) -> int:
+        raise NotImplementedError
+
+    def insert(self, item: BaseItem, pos: int) -> None:
+        raise NotImplementedError
+
+    def remove(self, item_id: str) -> None:
+        raise NotImplementedError
+
+    def reverse(self) -> None:
+        raise NotImplementedError
